import 'dart:convert';

import 'package:dd_taoke_sdk/model/brand_list_model.dart';
import 'package:dd_taoke_sdk/model/category.dart';
import 'package:dd_taoke_sdk/model/product.dart';
import 'package:dd_taoke_sdk/network/util.dart';
import 'package:dd_taoke_sdk/params/brand_param.dart';
import 'package:dd_taoke_sdk/params/brand_product_param.dart';
import 'package:dd_taoke_sdk/params/coupons_detail_link_param.dart';
import 'package:dd_taoke_sdk/params/high_commission_param.dart';
import 'package:dd_taoke_sdk/params/product_detail_param.dart';
import 'package:dd_taoke_sdk/params/product_list_param.dart';
import 'package:dd_taoke_sdk/params/speider_param.dart';
import 'package:flutter/cupertino.dart';

import 'model/brand_detail_result.dart';
import 'model/carousel_model.dart';
import 'model/coupon_link_result.dart';
import 'model/detail_base_data.dart';
import 'model/high_commission_result.dart';
import 'model/hot_search_worlds_result.dart';
import 'model/product_list_result.dart';
import 'model/product_material_result.dart';
import 'model/speider_resullt.dart';

class DdTaokeSdk {
  DdTaokeSdk._();

  static DdTaokeSdk get instance => DdTaokeSdk._();

  factory DdTaokeSdk() => instance;

  final util = DdTaokeUtil();

  /// 获取超级分类
  Future<List<Category>> getCategorys({ApiError error}) async {
    final response = await util.get('/categorys', error: error);
    return response.isNotEmpty ? categoryFromJson(response) : [];
  }

  /// 获取轮播图
  Future<List<Carousel>> getCarousel({ApiError error}) async {
    final response = await util.get('/carousel-list', error: error);
    return response.isNotEmpty ? carouselFromJson(response) : [];
  }

  /// 获取品牌列表
  Future<BrandListResult> getBrandList(
      {@required BrandListParam param, ApiError error}) async {
    final response =
        await util.get('/brand-list', data: param.toJson(), error: error);
    return response.isNotEmpty ? brandListResultFromJson(response) : null;
  }

  /// 获取商品列表
  Future<ProductListResult> getProducts(
      {@required ProductListParam param, ApiError error}) async {
    final response =
        await util.get('/goods', data: param.toJson(), error: error);
    return response.isNotEmpty ? productListFromJson(response) : null;
  }

  /// 获取商品详情
  Future<Product> getProductDetail(
      {@required ProductDetailParam param, ApiError error}) async {
    final response =
        await util.get('/detail', data: param.toJson(), error: error);
    return response.isNotEmpty ? productFromJson(response) : null;
  }

  /// 获取品牌详情
  Future<BrandDetail> getBrandDetail(
      {@required BrandProductParam param, ApiError error}) async {
    final response =
        await util.get('/brand-detail', data: param.toJson(), error: error);
    return response.isNotEmpty ? brandDetailFromJson(response) : null;
  }

  /// 高效转链
  /// [taobaoGoodsId] 是淘宝的商品id ,不是大淘客的id
  Future<CouponLinkResult> getCouponsDetail(
      {@required String taobaoGoodsId, ApiError error}) async {
    final response = await util.get('/privilege-link',
        data: CouponsDetailParam(goodsId: taobaoGoodsId).toJson(),
        error: error);
    print(response);
    return response.isNotEmpty
        ? CouponLinkResult.fromJson(jsonDecode(response))
        : null;
  }

  /// 获取商品详情页面所需的基本数据
  Future<DetailBaseDataResult> getDetailBaseData(
      {@required String productId}) async {
    final response = await util.get('/product-detail-all/$productId');
    var result;

    if (response.isNotEmpty) {
      final map = jsonDecode(response);
      final info = map['detail'];
      final products = map['similarList'];
      final coupon = map['privieleLink'];

      var list = <Product>[];
      for (final item in jsonDecode(products)) {
        list.add(Product.fromJson(item));
      }

      result = DetailBaseDataResult(
          info: Product.fromJson(jsonDecode(info)),
          similarProducts: list,
          couponInfo: CouponLinkResult.fromJson(jsonDecode(coupon)));
    }

    return result;
  }

  /// 获取高佣精选商品
  Future<HighCommissionResult> getHighCommissionProducts(
      {@required HighCommissionParam param, ApiError error}) async {
    final url = '/high-commission';
    final response = await util.get(url, data: param.toJson(), error: error);
    return response.isNotEmpty ? highCommissionResultFromJson(response) : null;
  }

  /// 获取商品的推广素材
  /// 商品精推素材
  /// [productId] 大淘客商品id或者淘宝商品id
  Future<ProductMaterialResult> getProductMaterial(
      {@required String productId, ApiError error}) async {
    final url = '/product-material';
    final response = await util.get(url, data: {"id": productId}, error: error);
    return response.isNotEmpty
        ? (productMaterialResultFromJson(response).isNotEmpty
            ? productMaterialResultFromJson(response)[0]
            : null)
        : [];
  }

  /// hot-search-worlds
  /// 热搜榜
  Future<List<HotSearchWorlds>> getHotSearchWorlds({ApiError error}) async {
    final url = '/hot-search-worlds';
    final response =await util.get(url, error: error);
    return response.isNotEmpty ? hotSearchWorldsFromJson(response) : [];
  }


  /// 获取线报
  Future<SpeiderResult> getSpeiderList({@required SpeiderParam param,ApiError error})async{
    final url = '/speider';
    final response = await util.get(url,data: param.toJson(),error: error);
    return response.isNotEmpty ? speiderResultFromJson(response) : null;
  }
}
